@@ -317,6 +317,21 @@ extern "C"
 				  unsigned int raw_flag, void *buf,
 				  size_t * sizeof_buf);
 
+  typedef void *gnutls_x509_dn_t;
+
+  typedef struct gnutls_x509_ava_st {
+    gnutls_datum_t oid;
+    gnutls_datum_t value;
+    unsigned long value_tag;
+  } gnutls_x509_ava_st;
+
+  int gnutls_x509_crt_get_subject (gnutls_x509_crt_t cert,
+				   gnutls_x509_dn_t *dn);
+  int gnutls_x509_crt_get_issuer (gnutls_x509_crt_t cert,
+				  gnutls_x509_dn_t *dn);
+  int gnutls_x509_dn_get_rdn_ava (gnutls_x509_dn_t dn, int irdn,
+				  int iava, gnutls_x509_ava_st *avast);
+
 /* CRL handling functions.
  */
   int gnutls_x509_crl_init (gnutls_x509_crl_t * crl);
