@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007 Free Software Foundation
+ * Copyright (C) 2002, 2003, 2004, 2005, 2006 Free Software Foundation
  *
  * Author: Nikos Mavroyanopoulos
  *
@@ -702,9 +702,9 @@ _gnutls_x509_raw_privkey_to_gkey (gnutls_privkey * privkey,
   return 0;
 }
 
-/* Reads a PEM encoded PKCS-1 RSA/DSA private key from memory.  Type
- * indicates the certificate format.  KEY can be NULL, to indicate
- * that GnuTLS doesn't know the private key.
+/* Reads a PEM encoded PKCS-1 RSA private key from memory
+ * 2002-01-26: Added ability to read DSA keys.
+ * type indicates the certificate format.
  */
 static int
 read_key_mem (gnutls_certificate_credentials_t res,
@@ -724,21 +724,16 @@ read_key_mem (gnutls_certificate_credentials_t res,
       return GNUTLS_E_MEMORY_ERROR;
     }
 
-  if (key)
-    {
-      tmp.data = (opaque *) key;
-      tmp.size = key_size;
+  tmp.data = (opaque *) key;
+  tmp.size = key_size;
 
-      ret =
-	_gnutls_x509_raw_privkey_to_gkey (&res->pkey[res->ncerts], &tmp, type);
-      if (ret < 0)
-	{
-	  gnutls_assert ();
-	  return ret;
-	}
+  ret =
+    _gnutls_x509_raw_privkey_to_gkey (&res->pkey[res->ncerts], &tmp, type);
+  if (ret < 0)
+    {
+      gnutls_assert ();
+      return ret;
     }
-  else
-    memset (&res->pkey[res->ncerts], 0, sizeof (gnutls_privkey));
 
   return 0;
 }
@@ -795,7 +790,7 @@ read_key_file (gnutls_certificate_credentials_t res,
   * gnutls_certificate_set_x509_key_mem - Used to set keys in a gnutls_certificate_credentials_t structure
   * @res: is an #gnutls_certificate_credentials_t structure.
   * @cert: contains a certificate list (path) for the specified private key
-  * @key: is the private key, or %NULL.
+  * @key: is the private key
   * @type: is PEM or DER
   *
   * This function sets a certificate/private key pair in the 
@@ -827,8 +822,7 @@ gnutls_certificate_set_x509_key_mem (gnutls_certificate_credentials_t
 
   /* this should be first 
    */
-  if ((ret = read_key_mem (res, key ? key->data : NULL,
-			   key ? key->size : 0, type)) < 0)
+  if ((ret = read_key_mem (res, key->data, key->size, type)) < 0)
     return ret;
 
   if ((ret = read_cert_mem (res, cert->data, cert->size, type)) < 0)
@@ -836,7 +830,7 @@ gnutls_certificate_set_x509_key_mem (gnutls_certificate_credentials_t
 
   res->ncerts++;
 
-  if (key && (ret = _gnutls_check_key_cert_match (res)) < 0)
+  if ((ret = _gnutls_check_key_cert_match (res)) < 0)
     {
       gnutls_assert ();
       return ret;
