@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003, 2004, 2005, 2006, 2007 Free Software Foundation
+ * Copyright (C) 2003, 2004, 2005, 2006 Free Software Foundation
  *
  * Author: Nikos Mavroyanopoulos
  *
@@ -164,14 +164,14 @@ is_issuer (gnutls_x509_crt_t cert, gnutls_x509_crt_t issuer_cert)
   NULL, 0};
   int ret;
 
-  ret = gnutls_x509_crt_get_raw_issuer_dn (cert, &dn1);
+  ret = _gnutls_x509_crt_get_raw_issuer_dn (cert, &dn1);
   if (ret < 0)
     {
       gnutls_assert ();
       goto cleanup;
     }
 
-  ret = gnutls_x509_crt_get_raw_dn (issuer_cert, &dn2);
+  ret = _gnutls_x509_crt_get_raw_dn (issuer_cert, &dn2);
   if (ret < 0)
     {
       gnutls_assert ();
@@ -541,7 +541,7 @@ _pkcs1_rsa_verify_sig (const gnutls_datum_t * text,
 		       const gnutls_datum_t * signature, mpi_t * params,
 		       int params_len)
 {
-  gnutls_mac_algorithm_t hash;
+  gnutls_mac_algorithm_t hash = GNUTLS_MAC_UNKNOWN;
   int ret;
   opaque digest[MAX_HASH_SIZE], md[MAX_HASH_SIZE];
   int digest_size;
@@ -899,7 +899,7 @@ is_crl_issuer (gnutls_x509_crl_t crl, gnutls_x509_crt_t issuer_cert)
       goto cleanup;
     }
 
-  ret = gnutls_x509_crt_get_raw_dn (issuer_cert, &dn2);
+  ret = _gnutls_x509_crt_get_raw_dn (issuer_cert, &dn2);
   if (ret < 0)
     {
       gnutls_assert ();
