@@ -1,7 +1,6 @@
 /*
  * Copyright (C) 2003, 2004, 2005, 2006, 2007 Free Software Foundation
- *
- * Author: Nikos Mavroyanopoulos
+ * Author: Nikos Mavroyanopoulos, Simon Josefsson, Howard Chu
  *
  * This file is part of GNUTLS.
  *
@@ -1718,6 +1717,144 @@ _gnutls_x509_crt_get_raw_dn (gnutls_x509_crt_t cert, gnutls_datum_t * start)
   return _gnutls_x509_crt_get_raw_dn2 (cert, "subject", start);
 }
 
+static int
+get_dn (gnutls_x509_crt_t cert, const char *whom, gnutls_x509_dn_t *dn)
+{
+  *dn = asn1_find_node(cert->cert, whom);
+  if (!*dn)
+    return GNUTLS_E_ASN1_ELEMENT_NOT_FOUND;
+  return 0;
+}
+
+/**
+ * gnutls_x509_crt_get_subject: get opaque subject DN pointer
+ * @cert: should contain a gnutls_x509_crt_t structure
+ * @dn: output variable with pointer to opaque DN.
+ *
+ * Return the Certificate's Subject DN as an opaque data type.  You
+ * may use gnutls_x509_dn_get_rdn_ava() to decode the DN.
+ *
+ * Returns: Returns 0 on success, or an error code.
+ **/
+int
+gnutls_x509_crt_get_subject (gnutls_x509_crt_t cert,
+				gnutls_x509_dn_t *dn)
+{
+  return get_dn (cert, "tbsCertificate.subject.rdnSequence", dn);
+}
+
+/**
+ * gnutls_x509_crt_get_issuer: get opaque issuer DN pointer
+ * @cert: should contain a gnutls_x509_crt_t structure
+ * @dn: output variable with pointer to opaque DN
+ *
+ * Return the Certificate's Issuer DN as an opaque data type.  You may
+ * use gnutls_x509_dn_get_rdn_ava() to decode the DN.
+ *
+ * Note that @dn points into the @cert object, and thus you may not
+ * deallocate @cert and continue to access @dn.
+ *
+ * Returns: Returns 0 on success, or an error code.
+ **/
+int
+gnutls_x509_crt_get_issuer (gnutls_x509_crt_t cert,
+			    gnutls_x509_dn_t *dn)
+{
+  return get_dn (cert, "tbsCertificate.issuer.rdnSequence", dn);
+}
+
+/**
+ * gnutls_x509_dn_get_rdn_ava:
+ * @dn: input variable with opaque DN pointer
+ * @irdn: index of RDN
+ * @iava: index of AVA.
+ * @ava: Pointer to structure which will hold output information.
+ *
+ * Get pointers to data within the DN.
+ *
+ * Note that @ava will contain pointers into the @dn structure, so you
+ * should not modify any data or deallocate it.  Note also that the DN
+ * in turn points into the original certificate structure, and thus
+ * you may not deallocate the certificate and continue to access @dn.
+ *
+ * Returns: Returns 0 on success, or an error code.
+ **/
+int
+gnutls_x509_dn_get_rdn_ava (gnutls_x509_dn_t dn,
+			    int irdn,
+			    int iava,
+			    gnutls_x509_ava_st *ava)
+{
+  ASN1_TYPE rdn, elem;
+  long len;
+  int lenlen, remlen, ret;
+  char rbuf[MAX_NAME_SIZE];
+  unsigned char cls, *ptr;
+
+  iava++; irdn++;			/* 0->1, 1->2 etc */
+
+  snprintf (rbuf, sizeof(rbuf), "rdnSequence.?%d.?%d", irdn, iava);
+  rdn = asn1_find_node (dn, rbuf);
+  if (!rdn)
+    {
+      gnutls_assert();
+      return GNUTLS_E_ASN1_ELEMENT_NOT_FOUND;
+    }
+
+  snprintf (rbuf, sizeof(rbuf), "?%d.type", iava);
+  elem = asn1_find_node (rdn, rbuf);
+  if (!elem)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_ASN1_ELEMENT_NOT_FOUND;
+    }
+
+  ava->oid.data = elem->value;
+  ava->oid.size = elem->value_len;
+
+  snprintf(rbuf, sizeof(rbuf), "?%d.value", iava);
+  elem = asn1_find_node(rdn, rbuf);
+  if (!elem)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_ASN1_ELEMENT_NOT_FOUND;
+    }
+
+  /* The value still has the previous tag's length bytes, plus the
+   * current value's tag and length bytes. Decode them.
+   */
+
+  ptr = elem->value;
+  remlen = elem->value_len;
+  len = asn1_get_length_der(ptr, remlen, &lenlen);
+  if (len < 0)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_ASN1_DER_ERROR;
+    }
+
+  ptr += lenlen;
+  remlen -= lenlen;
+  ret = asn1_get_tag_der(ptr, remlen, &cls, &lenlen, &ava->value_tag);
+  if (ret)
+    {
+      gnutls_assert ();
+      return _gnutls_asn2err (ret);
+    }
+
+  ptr += lenlen;
+  remlen -= lenlen;
+
+  ava->value.size = asn1_get_length_der(ptr, remlen, &lenlen);
+  if (ava->value.size < 0)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_ASN1_DER_ERROR;
+    }
+  ava->value.data = ptr + lenlen;
+
+  return 0;
+}
 
 /**
   * gnutls_x509_crt_get_fingerprint - This function returns the Certificate's fingerprint
@@ -1817,61 +1954,18 @@ gnutls_x509_crt_export (gnutls_x509_crt_t cert,
 }
 
 
-/**
-  * gnutls_x509_crt_get_key_id - Return unique ID of public key's parameters
-  * @crt: Holds the certificate
-  * @flags: should be 0 for now
-  * @output_data: will contain the key ID
-  * @output_data_size: holds the size of output_data (and will be
-  *   replaced by the actual size of parameters)
-  *
-  * This function will return a unique ID the depends on the public key
-  * parameters. This ID can be used in checking whether a certificate
-  * corresponds to the given private key.
-  *
-  * If the buffer provided is not long enough to hold the output, then
-  * *output_data_size is updated and GNUTLS_E_SHORT_MEMORY_BUFFER will
-  * be returned.  The output will normally be a SHA-1 hash output,
-  * which is 20 bytes.
-  *
-  * Return value: In case of failure a negative value will be
-  *   returned, and 0 on success.
-  *
-  **/
-int
-gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
-			    unsigned char *output_data,
-			    size_t * output_data_size)
+static int
+rsadsa_get_key_id (gnutls_x509_crt_t crt, int pk,
+		   unsigned char *output_data,
+		   size_t * output_data_size)
 {
   mpi_t params[MAX_PUBLIC_PARAMS_SIZE];
   int params_size = MAX_PUBLIC_PARAMS_SIZE;
-  int i, pk, result = 0;
+  int i, result = 0;
   gnutls_datum_t der = { NULL, 0 };
   GNUTLS_HASH_HANDLE hd;
 
-  if (crt == NULL)
-    {
-      gnutls_assert ();
-      return GNUTLS_E_INVALID_REQUEST;
-    }
-
-  if (*output_data_size < 20)
-    {
-      gnutls_assert ();
-      *output_data_size = 20;
-      return GNUTLS_E_SHORT_MEMORY_BUFFER;
-    }
-
-  pk = gnutls_x509_crt_get_pk_algorithm (crt, NULL);
-
-  if (pk < 0)
-    {
-      gnutls_assert ();
-      return pk;
-    }
-
   result = _gnutls_x509_crt_get_mpis (crt, params, &params_size);
-
   if (result < 0)
     {
       gnutls_assert ();
@@ -1914,7 +2008,7 @@ gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
 
   result = 0;
 
-cleanup:
+ cleanup:
 
   _gnutls_free_datum (&der);
 
@@ -1927,6 +2021,98 @@ cleanup:
   return result;
 }
 
+/**
+  * gnutls_x509_crt_get_key_id - Return unique ID of public key's parameters
+  * @crt: Holds the certificate
+  * @flags: should be 0 for now
+  * @output_data: will contain the key ID
+  * @output_data_size: holds the size of output_data (and will be
+  *   replaced by the actual size of parameters)
+  *
+  * This function will return a unique ID the depends on the public
+  * key parameters. This ID can be used in checking whether a
+  * certificate corresponds to the given private key.
+  *
+  * If the buffer provided is not long enough to hold the output, then
+  * *output_data_size is updated and GNUTLS_E_SHORT_MEMORY_BUFFER will
+  * be returned.  The output will normally be a SHA-1 hash output,
+  * which is 20 bytes.
+  *
+  * Return value: In case of failure a negative value will be
+  *   returned, and 0 on success.
+  *
+  **/
+int
+gnutls_x509_crt_get_key_id (gnutls_x509_crt_t crt, unsigned int flags,
+			    unsigned char *output_data,
+			    size_t * output_data_size)
+{
+  int pk, result = 0;
+  gnutls_datum_t pubkey;
+
+  if (crt == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_INVALID_REQUEST;
+    }
+
+  if (*output_data_size < 20)
+    {
+      gnutls_assert ();
+      *output_data_size = 20;
+      return GNUTLS_E_SHORT_MEMORY_BUFFER;
+    }
+
+  pk = gnutls_x509_crt_get_pk_algorithm (crt, NULL);
+  if (pk < 0)
+    {
+      gnutls_assert ();
+      return pk;
+    }
+
+  if (pk == GNUTLS_PK_RSA || pk == GNUTLS_PK_DSA)
+    {
+      /* This is for compatibility with what GnuTLS has printed for
+	 RSA/DSA before the code below was added.  The code below is
+	 applicable to all types, and it would probably be a better
+	 idea to use it for RSA/DSA too, but doing so would break
+	 backwards compatibility.  */
+      return rsadsa_get_key_id (crt, pk, output_data, output_data_size);
+    }
+
+  pubkey.size = 0;
+  result = asn1_der_coding (crt->cert, "tbsCertificate.subjectPublicKeyInfo",
+			    NULL, &pubkey.size, NULL);
+  if (result != ASN1_MEM_ERROR)
+    {
+      gnutls_assert ();
+      return _gnutls_asn2err (result);
+    }
+
+  pubkey.data = gnutls_alloca (pubkey.size);
+  if (pubkey.data == NULL)
+    {
+      gnutls_assert ();
+      return GNUTLS_E_MEMORY_ERROR;
+    }
+
+  result = asn1_der_coding (crt->cert, "tbsCertificate.subjectPublicKeyInfo",
+			    pubkey.data, &pubkey.size, NULL);
+  if (result != ASN1_SUCCESS)
+    {
+      gnutls_assert ();
+      gnutls_afree (pubkey.data);
+      return _gnutls_asn2err (result);
+    }
+
+  result = gnutls_fingerprint (GNUTLS_DIG_SHA1, &pubkey,
+			       output_data, output_data_size);
+
+  gnutls_afree (pubkey.data);
+
+  return result;
+}
+
 
 #ifdef ENABLE_PKI
 
