@@ -35,7 +35,6 @@
 #include <gnutls/extra.h>
 #include <gnutls/x509.h>
 #include <gnutls/openpgp.h>
-#include <gnutls/pkcs11.h>
 
 #include "error.h"
 #include "read-file.h"
@@ -1016,8 +1015,6 @@ srp_username_callback (gnutls_session session,
 static void
 init_global_tls_stuff (void)
 {
-  gnutls_x509_crt_t * ca_list;
-  unsigned int ncas;
   int ret;
 
   /* X509 stuff */
@@ -1032,18 +1029,6 @@ init_global_tls_stuff (void)
   gnutls_certificate_set_verify_flags (xcred,
 				       GNUTLS_VERIFY_ALLOW_X509_V1_CA_CRT);
 
-  ret = gnutls_pkcs11_get_ca_certificates (&ca_list, &ncas);
-  if (ret < 0)
-    {
-      fprintf (stderr, "Error getting CAs from PKCS#11\n");
-    }
-
-  ret = gnutls_certificate_set_x509_trust (xcred, ca_list, ncas);
-  if (ret < 0)
-    {
-      fprintf (stderr, "Error setting the PKCS#11 x509 trusts\n");
-    }
-
   if (x509_cafile != NULL)
     {
       ret =
